/*                      BIGPLAN.C                   */ 
/*                         by                       */ 
/*         Austin Kreisler and David Lovelock       */ 
/*                                                  */
/*    Modified by Kevin Sweere and Robert Reynolds  */ 
/*
 
   Because of the length of time required to decide how best to allocate 
resources, CASIE III will only do the computation if the number of resources 
is limited to 6, and the number of segments is limited to 10. 
   If you need to do more, and if you have a very fast computer, then 
below is the UNTESTED bare bones "C" code of the computation. 
 
   If you want to use this code, you should: 
 
1. Copy this file, and make sure it compiles on your machine. 
   This code has been compiled under Microsoft C, on DOS machines. 
   I have NO IDEA what problems will be encountered on any other machines. 
2. Run the program you have just compiled.  It may take a long time. 
   Check that the output is: 
 
                  ROW = 15.000000% 
                  Plan 1 
                  Team 1 in segment 1 
                  Team 2 in segment 5 
                  Team 3 in segment 1 
                  Team 4 in segment 2 
                  Team 5 in segment 3 
                  Team 6 in segment 4 
                  R.O.W. = 27.522936% 
                  Press RETURN when ready 
                  Plan 2 
                  Team 1 in segment 5 
                  Team 2 in segment 1 
                  Team 3 in segment 1 
                  Team 4 in segment 2 
                  Team 5 in segment 3 
                  Team 6 in segment 4 
                  R.O.W. = 27.397260% 
                  Press RETURN when ready 
                  Plan 3 
                  Team 1 in segment 5 
                  Team 2 in segment 5 
                  Team 3 in segment 1 
                  Team 4 in segment 2 
                  Team 5 in segment 3 
                  Team 6 in segment 4 
                  R.O.W. = 27.322404% 
                  Press RETURN when ready 
 
3. If you get this far, then rerun the program, and time it.  This test case 
   consists of 5 segments and 6 resources.  It requires the computer to check 
                          5*5*5*5*5*5 = 15,625 cases. 
   It takes about 90 seconds on a 386 machine. 
   From this you should be able to estimate how long your case will take. 
   For example, on a 386 machine, 10 segments and 11 resources would take 
                 11 
               10   x 90 / 15625 seconds = 18.26 years. 
   Recheck your estimate - this is not the time to say "Oops, sorry!" 
4. Redefine MAX_AREAS to the number of segments in your search area, 
   excluding the R.O.W. 
5. Redefine MAX_TEAMS to the number of resources available. 
6. Adjust the size of the team_pod matrix and then complete the team_pod 
   matrix according to the following scheme: 
      the first entry is the POD for resource 1 in segment 1, 
      the second entry is the POD for resource 1 in segment 2, etc. 
      All PODs should be between 0.0 and 1.0 - NOT PERCENTAGES. 
7. Adjust the size of the poa matrix and then complete the poa matrix 
   according to the following scheme: 
      the first entry is the current POA for segment 1, 
      the second entry is the current POA for segment 2, etc. 
      All POAs should be between 0.0 and 1.0 - NOT PERCENTAGES. 
8. Recompile the program. 
 
   WARNING - before running the real scenario, time a test case. 

*/ 
 
 
#include <stdio.h> 
#include <math.h> 
#include <time.h> 
#include <stdlib.h> 

#define FALSE (1==0)
#define TRUE (1==1)
 
/* change the following to fit your scenario */ 

#define  MAX_AREAS    5   /* excluding ROW */
#define  MAX_TEAMS    6

static   double   team_pod[MAX_TEAMS][MAX_AREAS]=
{
 0.2, 0.2, 0.2, 0.2, 0.2,  /* Team 1s POD for segments 1 to MAX_AREAS */
 0.3, 0.3, 0.0, 0.3, 0.3,  /* Team 2s POD for segments 1 to MAX_AREAS */
 0.5, 0.0, 0.0, 0.0, 0.0,  /* Team 3s POD for segments 1 to MAX_AREAS */
 0.0, 0.5, 0.0, 0.0, 0.0,  /* Team 4s POD for segments 1 to MAX_AREAS */
 0.0, 0.0, 0.6, 0.0, 0.0,  /* Team 5s POD for segments 1 to MAX_AREAS */
 0.0, 0.0, 0.0, 0.6, 0.0   /* MAX_TEAMs POD for segments 1 to MAX_AREAS */
};

static   double   poa[MAX_AREAS]=
{
 .25, .1, .2, .15, .15     /* Current POAs for segments 1 to MAX_AREAS */
};

static   double   team_pon[MAX_TEAMS][MAX_AREAS];

/* Don't change anything from here on (unless you find a bug!) */ 
 
int      where[3][MAX_TEAMS]; 
int      best[3];
double   row; 
double   hit[3]; 
 
void     main(); 
void     print_plan(); 
int      plan(); 
 
void 
main(void) 
{ 
   int   areas, teams, h, j; 
   int   return_value; 
 
   areas = MAX_AREAS; 
   teams = MAX_TEAMS; 
   for (h = 0 ; h < areas ; h++) 
      { 
      if ((poa[h] < 0.0) || (poa[h] > 1.0)) 
         { 
         printf("POA for segment %d must be between 0 and 1\n", h+1); 
         exit(1); 
         } 
      } 
   for (h = 0 ; h < areas ; h++) 
      { 
      for (j = 0 ; j < teams ; j++) 
         { 
         if ((team_pod[j][h] < 0.0) || (team_pod[j][h] > 1.0)) 
            { 
            printf("POD for team %d in segment %d must be between 0 and 1\n", j+1, h+1); 
            exit(2); 
            } 
         team_pon[j][h] = 1.0 - team_pod[j][h];
         } 
      } 
   row = 0.0; 
   for (h = 0 ; h < areas ; h++) 
      row = row + poa[h]; 
   row = 1.0 - row; 
   if (row <= 0.0) 
      { 
      printf("ROW POA must be positive\n"); 
      exit(3); 
      } 
   else 
      printf("ROW = %lf%%\n", row*100.0); 
   return_value = plan(areas, teams); 
   if (return_value == -1) 
      { 
      printf("Too many teams"); 
      exit(1); 
      } 
   print_plan(teams, row); 
   exit(0); 
} 
 
void 
print_plan(int teams, double row) 
{ 
   int   h; 
 
   printf("Plan 1\n"); 
   for (h = 0 ; h < teams ; h++) 
      { 
      printf("Team %d in segment %d\n", h+1, where[best[0]][h]+1); 
      } 
   printf("R.O.W. = %lf%%\nPress RETURN when ready\n", row*100.0/(1.0-hit[best[0]])); 
   getchar(); 
   printf("Plan 2\n"); 
   for (h = 0 ; h < teams ; h++) 
      { 
      printf("Team %d in segment %d\n", h+1, where[best[1]][h]+1); 
      } 
   printf("R.O.W. = %lf%%\nPress RETURN when ready\n", row*100.0/(1.0-hit[best[1]])); 
   getchar(); 
   printf("Plan 3\n"); 
   for (h = 0 ; h < teams ; h++) 
      { 
      printf("Team %d in segment %d\n", h+1, where[best[2]][h]+1); 
      } 
   printf("R.O.W. = %lf%%\nPress RETURN when ready\n", row*100.0/(1.0-hit[best[2]])); 
   getchar(); 
} 
 
int 
plan(int segment, int teams) 
{ 
   int      h, g, k; 
   int      j[MAX_TEAMS]; 
   double   cpon[MAX_AREAS]; 
   double   sum; 
   int      index; 
   int      prev;
   int      done;
   int      new_best;
 
   if (teams > MAX_TEAMS) 
      { 
      return -1; 
      } 
   hit[0] = hit[1] = hit[2] = 0.0; 
 
   for (index=0; index<teams; index++)
      j[index] = 0;
   for (h = 0 ; h < segment ; h++) 
      cpon[h] = 1.0; 
   for(g = 0; g < teams; g++) 
      cpon[j[g]] *= team_pon[g][j[g]]; 
   sum = 0.0; 
   for (h = 0 ; h < segment ; h++) 
      sum = sum + (1.0 - cpon[h])*poa[h]; 
   best[0] = 0;
   best[1] = 1;
   best[2] = 2;
   hit[best[0]] = sum;
   done = FALSE;
   while (!done)
      { 
      index = 0;
      while (index < teams)
         {
         prev = j[index];
         if (++j[index] >= segment)
            j[index] = 0;
         cpon[prev] /= team_pon[index][prev];
         sum -= team_pod[index][prev] * cpon[prev] * poa[prev];
         sum += team_pod[index][j[index]] * cpon[j[index]] * poa[j[index]];
         cpon[j[index]] *= team_pon[index][j[index]];
         if (j[index] != 0)
            break;
         else
            index++;
         }
      if (index >= teams)
         {
         done = TRUE;
         continue;
         }
      if (sum > hit[best[0]])
         {
         new_best = best[2];
         hit[new_best] = sum;
         for (g = 0; g < teams; g++) 
            where[new_best][g] = j[g]; 
         best[2] = best[1];
         best[1] = best[0];
         best[0] = new_best;
         }
      else if (sum > hit[best[1]])
         {
         new_best = best[2];
         hit[new_best] = sum;
         for (g = 0; g < teams; g++) 
            where[new_best][g] = j[g]; 
         best[2] = best[1];
         best[1] = new_best;
         }
      else if (sum > hit[best[2]])
         {
         hit[best[2]] = sum;
         for (g = 0; g < teams; g++) 
            where[best[2]][g] = j[g]; 
         }
      } 
   return 1; 
} 
